package com.css.simulation.resource.scheduler.common.resource;

import api.common.pojo.dto.ProjectMessageDTO;
import api.common.util.JsonUtil;
import com.css.simulation.resource.scheduler.service.domain.PrefixTO;
import com.css.simulation.resource.scheduler.common.util.ProjectUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

@Component
@Slf4j
public class TaskLock {
    private final Lock lock = new ReentrantLock();

    @Resource
    StringRedisTemplate stringRedisTemplate;
    @Resource
    ProjectUtil projectUtil;

    /**
     * 判断完成度
     */
    public boolean complete(
            PrefixTO redisPrefix,
            String projectId,
            String nodeName,
            String podName
    ) {
        boolean result = false;
        lock.lock();
        try {
            ProjectMessageDTO projectMessageDTO = JsonUtil.jsonToBean(stringRedisTemplate.opsForValue().get(redisPrefix.getProjectRunningKey()), ProjectMessageDTO.class);
            int taskTotal = projectMessageDTO.getTaskTotal();
            int taskCompleted = projectMessageDTO.getTaskCompleted();
            log.info("complete() 项目 " + projectId + " 完成进度为：" + (taskCompleted + 1) + "/" + taskTotal);
            if (taskCompleted + 1 == taskTotal) {
                result = true;
            } else {    // 项目没有完成
                projectMessageDTO.setTaskCompleted(taskCompleted + 1);  // 增加已完成任务数
                stringRedisTemplate.opsForValue().set(redisPrefix.getProjectRunningKey(), JsonUtil.beanToJson(projectMessageDTO));
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            lock.unlock();
        }
        return result;
    }
}
