package com.css.simulation.resource.scheduler.dao.manager;

import api.common.pojo.constants.DictConstants;
import api.common.util.FileUtil;
import com.css.simulation.resource.scheduler.common.configuration.kubernetes.KubernetesConfiguration;
import com.css.simulation.resource.scheduler.common.configuration.minio.MinioConfiguration;
import com.css.simulation.resource.scheduler.dao.mapper.VehicleMapper;
import com.css.simulation.resource.scheduler.common.util.ProjectUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;

@Component
@Slf4j
public class ProjectManager {

    @Value("${scheduler.linux-path.vtd-pod-template-yaml}")
    private String vtdPodTemplateYaml;
    @Value("${scheduler.linux-path.carsim-pod-template-yaml}")
    private String carsimPodTemplateYaml;
    @Value("${scheduler.simulation-cloud-ip}")
    private String simulationCloudIp;
    @Value("${scheduler.linux-path.pod-yaml-directory}")
    private String podYamlDirectory;
    @Value("${spring.kafka.bootstrap-servers}")
    private String kafkaIp;
    @Resource
    private KubernetesConfiguration kubernetesConfiguration;
    @Resource
    private MinioConfiguration minioConfiguration;
    @Resource
    private ProjectUtil projectUtil;
    @Resource
    private VehicleMapper vehicleMapper;
    @Resource
    private StringRedisTemplate stringRedisTemplate;


    /**
     * 创建一个临时 yaml，node 在最后用 # 号隔开
     */
    @SneakyThrows
    public String createTempYaml(String projectId,
                                 String vehicleConfigId,
                                 String modelType,
                                 String algorithmDockerImage,
                                 String nodeName,
                                 int kafkaPartition,
                                 long kafkaOffset,
                                 String isChoiceGpu,
                                 Integer cpuOrder

    ) {
        String podName = projectUtil.getRandomPodName(projectId);   // 生成 podName
        String podYaml = projectUtil.getPodYamlName(nodeName, podName);     // 模板文件名称
        String yamlPath = podYamlDirectory + podYaml;
        String finalYaml;

        if ("1".equals(modelType)) {
            String podString = FileUtil.read(new File(vtdPodTemplateYaml));
            String replace0 = podString.replace("vtd-container", "vtd-" + projectId);
            String replace1 = replace0.replace("simulation-cloud-ip", simulationCloudIp);
            String replace2 = replace1.replace("kafka-ip", kafkaIp);
            String replace3 = replace2.replace("kafka-topic", projectId);     // 消息主题名称为 projectId
            String replace4 = replace3.replace("kafka-partition", "\"" + kafkaPartition + "\"");
            String replace5 = replace4.replace("kafka-offset", "\"" + kafkaOffset + "\"");
            String replace6 = replace5.replace("minio-ip", minioConfiguration.getEndpointWithoutHttp());
            String replace7 = replace6.replace("minio-access-key", minioConfiguration.getAccessKey());
            String replace8 = replace7.replace("minio-secret-key", minioConfiguration.getSecretKey());

            String replace9 = replace8.replace("algorithm-container", "algorithm-" + projectId);
            String replace10 = replace9.replace("algorithm-image", algorithmDockerImage);

            String replace11 = replace10.replace("pod-name", podName); // pod 名称包括 projectId 和 随机字符串
            String replace12 = replace11.replace("namespace-name", kubernetesConfiguration.getNamespace()); // pod 名称包括 projectId 和 随机字符串
            String replace13 = replace12.replace("node-name", nodeName);     // 指定 pod 运行节点

            String replace14;
            if (cpuOrder != null) {
                replace14 = replace13.replace("cpu-order", "\"" + cpuOrder + "\"");     // 指定 cpu 编号
                stringRedisTemplate.opsForValue().set("project:" + projectId + ":pod:" + podName + ":cpu", cpuOrder + "");    //  pod 运行使用的 cpu编号
            } else {
                replace14 = replace13;
            }

            if (DictConstants.USE_GPU.equals(isChoiceGpu)) {
                String replace15 = replace14.replace("vtd-image", kubernetesConfiguration.getImageVtdGpu());
                finalYaml = replace15.replace("vtd-command", kubernetesConfiguration.getCommandVtdGpu());
            } else if (DictConstants.NOT_USE_GPU.equals(isChoiceGpu)) {
                String replace15 = replace14.replace("vtd-image", kubernetesConfiguration.getImageVtdNogpu());
                finalYaml = replace15.replace("vtd-command", kubernetesConfiguration.getCommandVtdNogpu());
            } else {
                throw new RuntimeException("createTempYaml() 是否使用 gpu：" + isChoiceGpu);
            }
        } else if ("2".equals(modelType)) {
            String podString = FileUtil.read(new File(carsimPodTemplateYaml));
            String replace0 = podString.replace("vtd-container", "vtd-" + projectId);
            String replace1 = replace0.replace("simulation-cloud-ip", simulationCloudIp);
            String replace2 = replace1.replace("kafka-ip", kafkaIp);
            String replace3 = replace2.replace("kafka-topic", projectId);     // 消息主题名称为 projectId
            String replace4 = replace3.replace("kafka-partition", "\"" + kafkaPartition + "\"");
            String replace5 = replace4.replace("kafka-offset", "\"" + kafkaOffset + "\"");
            String replace6 = replace5.replaceAll("minio-ip", minioConfiguration.getEndpointWithoutHttp());
            String replace7 = replace6.replaceAll("minio-access-key", minioConfiguration.getAccessKey());
            String replace8 = replace7.replaceAll("minio-secret-key", minioConfiguration.getSecretKey());

            String replace9 = replace8.replace("algorithm-container", "algorithm-" + projectId);
            String replace10 = replace9.replace("algorithm-image", algorithmDockerImage);

            String replace11 = replace10.replace("carsim-container", "carsim-" + projectId);
            String replace12 = replace11.replace("carsim-image", kubernetesConfiguration.getCarsimImage());
            String replace13 = replace12.replace("carsim-command", kubernetesConfiguration.getCarsimCommand());
            String replace14 = replace13.replace("minio-bucket", minioConfiguration.getBucketName());
            String replace15 = replace14.replace("par-path", vehicleMapper.selectParPathByVehicleConfigId(vehicleConfigId));

            String replace16 = replace15.replace("pod-name", podName); // pod 名称包括 projectId 和 随机字符串
            String replace17 = replace16.replace("namespace-name", kubernetesConfiguration.getNamespace()); // pod 名称包括 projectId 和 随机字符串
            String replace18 = replace17.replace("node-name", nodeName);     // 指定 pod 运行节点

            String replace19;
            if (cpuOrder != null) {
                replace19 = replace18.replace("cpu-order", "\"" + cpuOrder + "\"");     // 指定 cpu 编号
                stringRedisTemplate.opsForValue().set("project:" + projectId +":pod:" + podName + ":cpu", cpuOrder + "");    //  pod 运行使用的 cpu编号
            } else {
                replace19 = replace18;
            }

            if (DictConstants.USE_GPU.equals(isChoiceGpu)) {
                log.info("createTempYaml() k8s参数为：" + kubernetesConfiguration);
                log.info("createTempYaml() yaml模板为：" + replace12);
                String replace20 = replace19.replace("vtd-image", kubernetesConfiguration.getImageVtdGpu());
                finalYaml = replace20.replace("vtd-command", kubernetesConfiguration.getCommandVtdCarsimGpu());
            } else if (DictConstants.NOT_USE_GPU.equals(isChoiceGpu)) {
                String replace20 = replace19.replace("vtd-image", kubernetesConfiguration.getImageVtdNogpu());
                finalYaml = replace20.replace("vtd-command", kubernetesConfiguration.getCommandVtdCarsimNogpu());
            } else {
                throw new RuntimeException("createTempYaml() 是否使用 gpu：" + isChoiceGpu);
            }
        } else {
            throw new RuntimeException("createTempYaml() 模型类型错误：" + modelType);
        }
        log.info("保存项目 " + projectId + " 的 yaml 文件：" + yamlPath);
        FileUtil.writeStringToLocalFile(finalYaml, yamlPath);
        String yamlRedisKey = "project:" + projectId + ":node:" + nodeName + ":yaml:" + podName;
        stringRedisTemplate.opsForValue().set(yamlRedisKey, yamlPath);
        return yamlRedisKey;
    }
}
