package com.css.simulation.resource.scheduler.service;

import api.common.util.SshUtil;
import com.css.simulation.resource.scheduler.dao.manager.TaskManager;
import com.css.simulation.resource.scheduler.dao.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.dao.entity.ProjectPO;
import com.css.simulation.resource.scheduler.dao.entity.TaskPO;
import com.css.simulation.resource.scheduler.service.domain.PrefixTO;
import com.css.simulation.resource.scheduler.common.util.ProjectUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.sshd.client.SshClient;
import org.apache.sshd.client.session.ClientSession;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
@Slf4j
public class TaskService {

    @Value("${scheduler.host.hostname}")
    String hostname;
    @Value("${scheduler.host.username}")
    String username;
    @Value("${scheduler.host.password}")
    String password;
    @Resource
    TaskManager taskManager;
    @Resource
    TaskMapper taskMapper;
    @Resource
    ProjectUtil projectUtil;

    // -------------------------------- Comment --------------------------------

    @SneakyThrows
    public void taskState(String taskId, String state, String podName) {
        log.info("接收到参数为：taskId=" + taskId + "，state=" + state + "，podName=" + podName);
        TaskPO taskPO = taskMapper.selectById(taskId);
        if (taskPO == null) {
            log.error("接收到已删除但还在执行的任务：taskId=" + taskId + "，state=" + state + "，podName=" + podName);
            return;
        }
        String projectId = taskPO.getPId(); // 项目 id
        ProjectPO projectPO = projectUtil.getProjectByProjectId(projectId);
        String projectType = projectPO.getProjectType();  // 项目类型
        String maxSimulationTime = projectPO.getMaxSimulationTime();  // 项目类型
        String userId = taskPO.getCreateUserId();   // 用户 id
        PrefixTO redisPrefix = projectUtil.getRedisPrefixByUserIdAndProjectIdAndTaskId(userId, projectId, taskId);  // 项目前缀
        //1 判断项目是否已完成
        boolean projectCompleted = taskManager.isProjectCompleted(redisPrefix, projectId, projectType, maxSimulationTime, taskId, state, podName);
        if (!projectCompleted) {
            return;
        }
        SshClient sshClient = SshUtil.getClient();
        ClientSession clientSession = SshUtil.getSession(sshClient, hostname, username, password);
        //2 准备打分
        log.info("项目 " + projectId + " 准备打分。");
        taskManager.prepareScore(redisPrefix.getProjectRunningKey());
        //3 打分
        taskManager.score(userId, projectId, projectType, clientSession);
        //4 调用 server 的接口，计算评价等级
        taskManager.evaluationLevel(projectId);
        //5 结束
        taskManager.done(redisPrefix, sshClient, clientSession, projectId, projectType);

    }


    public Boolean taskConfirm(String taskId) {
        return taskManager.taskConfirm(taskId);
    }

    public void taskTick(String taskId) {
        taskManager.taskTick(taskId);
    }


}



