package com.css.simulation.resource.scheduler.web.scheduler;

import api.common.util.SshUtil;
import com.css.simulation.resource.scheduler.common.configuration.docker.DockerConfiguration;
import com.css.simulation.resource.scheduler.dao.mapper.AlgorithmMapper;
import com.css.simulation.resource.scheduler.dao.entity.AlgorithmPO;
import com.css.simulation.resource.scheduler.service.domain.DockerNodeTO;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.sshd.client.SshClient;
import org.apache.sshd.client.session.ClientSession;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Component
@Slf4j
public class AlgorithmScheduler {

    @Resource
    DockerConfiguration dockerConfiguration;
    @Resource
    AlgorithmMapper algorithmMapper;


    /**
     * 定时删除算法镜像
     * TODO 算法平台暂时手动删除
     */
    @Scheduled(fixedDelay = 60 * 60 * 1000)
    @SneakyThrows
    public void removeAlgorithmImage() {
        int maxAlgorithmImage = dockerConfiguration.getMaxAlgorithmImage();
        int minAlgorithmImage = dockerConfiguration.getMinAlgorithmImage();
        //1 查询已经导入的算法，根据创建时间排序
        List<AlgorithmPO> algorithmImportedList = algorithmMapper.selectByDockerImport("1");
        int algorithmImportNumber = algorithmImportedList.size();
        log.info("AlgorithmScheduler--removeAlgorithmImage kubernetes 各个节点已经导入 " + algorithmImportNumber + " 个算法。");
        if (algorithmImportNumber < maxAlgorithmImage) {
            return;
        }
        //2 如果已经导入到 docker 中的算法镜像已经达到 maxAlgorithmImage 个，则清理算法镜像到 minAlgorithmImage 个；
        List<String> algorithmIdToUpdateList = new ArrayList<>();
        StringBuilder dockerRmiCommand = new StringBuilder("docker rmi ");
        StringBuilder registryRmCommand = new StringBuilder("rm -rf ");
        for (int i = minAlgorithmImage; i < algorithmImportedList.size(); i++) {
            AlgorithmPO tempAlgorithm = algorithmImportedList.get(i);
            String tempAlgorithmId = tempAlgorithm.getId();
            String algorithmCode = tempAlgorithm.getAlgorithmCode();
            algorithmIdToUpdateList.add(tempAlgorithm.getId());
            dockerRmiCommand.append(tempAlgorithm.getDockerImage());
            registryRmCommand.append(tempAlgorithmId).append("_").append(algorithmCode);
        }
        //3 修改数据库为未导入
        algorithmMapper.updateDockerImportAndDockerImageByIdList("0", "", algorithmIdToUpdateList);
        //4 删除镜像
        List<DockerNodeTO> nodeList = dockerConfiguration.getNodeList();
        for (DockerNodeTO dockerNodeTO : nodeList) {
            String hostname = dockerNodeTO.getHostname();
            String username = dockerNodeTO.getUsername();
            String password = dockerNodeTO.getPassword();
            SshClient client = SshUtil.getClient();
            ClientSession session = SshUtil.getSession(client, hostname, username, password);
            SshUtil.execute(session, dockerRmiCommand.toString());
            SshUtil.stop(client, session);
        }
    }
}
