package com.css.simulation.resource.scheduler.web.scheduler;

import api.common.util.CollectionUtil;
import com.css.simulation.resource.scheduler.common.util.ProjectUtil;
import com.css.simulation.resource.scheduler.common.util.RedisUtil;
import com.css.simulation.resource.scheduler.web.consumer.ProjectConsumer;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

@Component
@Slf4j
public class ProjectScheduler {
    @Resource
    private StringRedisTemplate stringRedisTemplate;
    @Resource
    private ProjectConsumer projectConsumer;
    @Resource
    private ProjectUtil projectUtil;

    /**
     * 调度项目启动
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void dispatchProject() {
        List<String> projectMessageKeys = projectUtil.getWaitingProjectMessageKeys();
        if (!CollectionUtil.isEmpty(projectMessageKeys)) {
            log.info("尝试启动等待中的项目：" + projectMessageKeys);
            for (String projectMessageKey : projectMessageKeys) {
                final String projectMessage = RedisUtil.getStringByKey(stringRedisTemplate, projectMessageKey);
                ConsumerRecord<String, String> projectRecord = new ConsumerRecord<>("", 0, 0L, null, projectMessage);
                projectConsumer.acceptMessage(projectRecord);
            }
        }
    }

}
